/* ox_play.c */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "ox_toolkit.h"

OXFILE *fd_rw;

#define INIT_S_SIZE 2048
#define EXT_S_SIZE  2048

static int stack_size = 0;
static int stack_pointer = 0;
static cmo **stack = NULL;

int initialize_stack()
{
    stack_pointer = 0;
    stack_size = INIT_S_SIZE;
    stack = malloc(stack_size*sizeof(cmo*));
}

static int extend_stack()
{
    int size2 = stack_size + EXT_S_SIZE;
    cmo **stack2 = malloc(size2*sizeof(cmo*));
    memcpy(stack2, stack, stack_size*sizeof(cmo *));
    free(stack);
    stack = stack2;
    stack_size = size2;
}

int push(cmo* m)
{
    stack[stack_pointer] = m;
    stack_pointer++;
    if (stack_pointer >= stack_size) {
        extend_stack();
    }
}

cmo* pop()
{
    if (stack_pointer > 0) {
        stack_pointer--;
        return stack[stack_pointer];
    }
    return new_cmo_null();
}

void pops(int n)
{
    stack_pointer -= n;
    if (stack_pointer < 0) {
        stack_pointer = 0;
    }
}

#define OX_XSAMPLE_VERSION 0x11121400
#define ID_STRING  "1999/12/14 15:25:00"

int sm_mathcap()
{
  mathcap_init(OX_XSAMPLE_VERSION,ID_STRING,"ox_play",NULL,NULL);
  push(oxf_cmo_mathcap(fd_rw));
  return 0;
}

int sm_popCMO()
{
    cmo* m = pop();

    if (m != NULL) {
        send_ox_cmo(fd_rw, m);
        return 0;
    }
    return SM_popCMO;
}

cmo_error2 *make_error2(int code)
{
    return new_cmo_int32(-1);
}

int get_i()
{
    cmo *c = pop();
    if (c->tag == CMO_INT32) {
        return ((cmo_int32 *)c)->i;
    }else if (c->tag == CMO_ZZ){
      return mpz_get_si(((cmo_zz *)c)->mpz);
    }
    make_error2(-1);
    return 0;
}

int get_time()
{
  int hour;
  int min;
  int sec;
  time_t orgtime;
  struct tm *lpmytm;
  
  hour = get_i();
  pop();
  min = get_i(); 
  sec = get_i();
  
  if((hour < 0) || (hour > 24) || (min < 0) 
	 ||(min > 60 ) || (sec < 0 ) || (sec > 60 )){
    printf("\nstart\n");
  }
  else{
    printf("\nstart %d:%d:%d\n",hour,min,sec);

    while(1){
      orgtime = time(NULL);
      lpmytm = localtime(&orgtime);
      if((hour == lpmytm->tm_hour) 
		 && (min == lpmytm->tm_min) && (sec == lpmytm->tm_sec)){
		break;
      }
    }
  }
  return 0;
}


int my_play( char *s)
{
  if (play(s) != 0){
    push(make_error2(-1));
    return -1;
  }
  else{
    push(new_cmo_int32(0));
    return 0;
  }
}



int sm_executeFunction()
{
  get_time();
  cmo_string *sound_file = (cmo_string *)pop();
  if (sound_file->tag != CMO_STRING) {
    push(make_error2(0));
    return -1;
  }
  printf("%s\n",sound_file->s);
  my_play(sound_file->s);
}

	    

int receive_and_execute_sm_command()
{
    int code = receive_int32(fd_rw);
    switch(code) {
    case SM_popCMO:
	printf("SM_popCMO\n");
	sm_popCMO();
        break;
    case SM_executeFunction:
      	printf("SM_executeFunction\n");
        sm_executeFunction();
        break;
    case SM_mathcap:
	printf("SM_mathcap\n");
        sm_mathcap();
        break;
    case SM_setMathCap:
	printf("SM_setMathCap\n");
        pop();
        break;
    default:
		;
    }
}

int receive()
{
    int tag;
    tag = receive_ox_tag(fd_rw);
    switch(tag) {
    case OX_DATA:
      printf("Got OX_DATA\n");
        push(receive_cmo(fd_rw));
        break;
    case OX_COMMAND:
      printf("Got OX_COMMAND  ");
        receive_and_execute_sm_command();
        break;
    default:
		;
    }
    return 0;
}

int main()
{
    ox_stderr_init(stderr);
    initialize_stack();

	fd_rw = oxf_open(3);
	oxf_determine_byteorder_server(fd_rw);
       
    while(1) {
        receive();
    }
}


/* ox_play.c  end */
